import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class PeriodicTableDAO {
	private PeriodicTableDAO() { 
	}
	
	private static PeriodicTableDAO instance = new PeriodicTableDAO();
	public static PeriodicTableDAO getInstance(){
		return instance;
	} 

	// insert
	public int insert(Connection conn, PeriodicElementBean pe) throws SQLException {
		PreparedStatement pstmt = null;
		try {
			String sql = "insert into ptable values (?,?,?,?,?,?,?,?,?,?)";
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, pe.getNumber());
			pstmt.setString(2, pe.getName());
			pstmt.setString(3, pe.getSymbol());
			pstmt.setDouble(4, pe.getWeight());
			pstmt.setInt(5, pe.getNumberOfNeutrons());
			pstmt.setInt(6, pe.getNumberOfProtons());
			pstmt.setInt(7, pe.getNumberOfElectrons());
			pstmt.setInt(8, pe.getPeriod());
			pstmt.setInt(9, pe.getGroup());
			pstmt.setString(10, pe.getPhase().toString());
			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}

	// delete
	public int delete(Connection conn, int number) throws SQLException {
		PreparedStatement pstmt = null;
		try {
			String sql = "delete from ptable where number=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, number);
			return pstmt.executeUpdate();
		} finally {
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}

	// edit
	public int edit(Connection conn, PeriodicElementBean pe) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "update ptable set name=?, symbol=?, weight=?, numberOfNeutrons=?, numberOfProtons=?, numberOfNeutrons=?, numberOfElectrons=?, period=?, group=?, phase=? where number=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setString(1, pe.getName());
			pstmt.setString(2, pe.getSymbol());
			pstmt.setDouble(3, pe.getWeight());
			pstmt.setInt(4, pe.getNumberOfNeutrons());
			pstmt.setInt(5, pe.getNumberOfProtons());
			pstmt.setInt(6, pe.getNumberOfElectrons());
			pstmt.setInt(7, pe.getPeriod());
			pstmt.setInt(8, pe.getGroup());
			pstmt.setString(9, pe.getPhase().toString());
			pstmt.setInt(10, pe.getNumber());
			return pstmt.executeUpdate();
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	// select(find/get)
	public PeriodicElementBean select(Connection conn, int number) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from ptable where number=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, number);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return createFromResultSet(rs);
			}
			else {
				return null;
			}
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	public PeriodicElementBean createFromResultSet(ResultSet rs) throws SQLException {
		int number = rs.getInt("number");
		String name = rs.getString("name");
		String symbol = rs.getString("symbol");
		double weight = rs.getDouble("weight");
		int numberOfNeutrons = rs.getInt("numberOfNeutrons");
		int numberOfProtons = rs.getInt("numberOfProtons");
		int numberOfElectrons = rs.getInt("numberOfElectrons");
		int period = rs.getInt("period");
		int group = rs.getInt("group");
		Phase phase = Phase.nameOf(rs.getString("phase"));
		PeriodicElementBean pe = new PeriodicElementBean(number, name, symbol, weight, numberOfNeutrons,
				numberOfProtons, numberOfElectrons, period, group, phase);
		return pe;
	}
	
	// selectList
	public List<PeriodicElementBean> selectList(Connection conn) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from ptable";
			pstmt = conn.prepareStatement(sql);
			rs = pstmt.executeQuery();
			List<PeriodicElementBean> pList = new ArrayList<>();
			while (rs.next()) {
				pList.add(createFromResultSet(rs));
			}
			return pList;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	// selectListByPeriod
	public List<PeriodicElementBean> selectListByPeriod(Connection conn, int period) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from ptable where period=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, period);
			rs = pstmt.executeQuery();
			List<PeriodicElementBean> pList = new ArrayList<>();
			while (rs.next()) {
				pList.add(createFromResultSet(rs));
			}
			return pList;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
	
	// selectListByGroup
	public List<PeriodicElementBean> selectListByGroup(Connection conn, int group) throws SQLException {
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			String sql = "select * from ptable where group=?";
			pstmt = conn.prepareStatement(sql);
			pstmt.setInt(1, group);
			rs = pstmt.executeQuery();
			List<PeriodicElementBean> pList = new ArrayList<>();
			while (rs.next()) {
				pList.add(createFromResultSet(rs));
			}
			return pList;
		} finally {
			if (rs != null) {
				rs.close();
			}
			if (pstmt != null) {
				pstmt.close();
			}
		}
	}
}
