import java.sql.*;
import java.util.ArrayList;
import java.util.List;

public class SampleDBPTableTest {
	   
	public static void main(String[] args) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		String command = null;
		
		try {
			conn = ConnectionProvider.getConnection(); // getConnection
			PeriodicTableDAO dao = PeriodicTableDAO.getInstance(); // getInstance

			List<PeriodicElementBean> allList = dao.selectList(conn); // selectList
			System.out.println("print all list");
			allList.forEach(System.out::println);

			List<PeriodicElementBean> period3List = dao.selectListByPeriod(conn, 3); // selectList
			System.out.println("print only period3 list");
			period3List.forEach(System.out::println);

			// Execute a query
			System.out.println("Creating statement...");
			command = "select * from ptable where phase='GAS'";
			pstmt = conn.prepareStatement(command);
			// Extract data from result set
			ResultSet rs = pstmt.executeQuery("SELECT * from ptable where phase='GAS'");
			List<PeriodicElementBean> gasList = new ArrayList<>();
			while (rs.next()) {
				gasList.add(dao.createFromResultSet(rs));
			}
			System.out.println("print only gasList");
			gasList.forEach(System.out::println);

			// Clean-up environment
			pstmt.close();
			conn.close();
		} catch(SQLException se){
			//Handle errors for JDBC
			se.printStackTrace();
		} finally {
			//finally block used to close resources
			try {
				if(pstmt!=null) pstmt.close();
			} catch(SQLException se2){
			}// nothing we can do
			try {
				if(conn!=null) conn.close();
			} catch(SQLException se){
				se.printStackTrace();
			}//end finally try

		}//end try
		System.out.println("Goodbye!");
	} //end main


}
